@@ -1,15 +1,21 @@
 #ifndef _UTILNT_H
 #define _UTILNT_H
 
+#include <sys/types.h>
+#include "gettimeofday.h"
+
 char *strtok_r(char *s1, char *s2, char **lasts);
 double drand48();
 int getopt(int nargc, char * const *nargv, const char *ostr);
 
+/* for WinNT to startup winsock2 by Akira 12/27/01 */
+void startupSocket(void);
+
 /*
  * gettimeofday() function
  * Used in gettimeofdayf.c, error.c
  */
-int gettimeofday(struct timeval *tp, void * );
+//int gettimeofday(struct timeval *tp, void * );
 
 /*
  * For syslog, openlog, closelog functions
@@ -27,10 +33,11 @@ void syslog(int priority, const char *message, ... /* arguments */);
  * struct group *getgrnam(const char *name) funciton
  * Used in gname2id.c
  */
-typedef long    pid_t;					/* PID type				*/
-typedef long    uid_t;                  /* UID type             */
-typedef uid_t   gid_t;                  /* GID type             */
-
+#ifndef __BORLANDC__
+typedef long    pid_t;                  /* PID type       */
+typedef long    uid_t;                  /* UID type       */
+typedef uid_t   gid_t;                  /* GID type       */
+#endif
 struct  group { /* see getgrent(3) */
         char    *gr_name;
         char    *gr_passwd;
@@ -82,7 +89,7 @@ struct passwd *getpwnam(const char *name);
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: utilnt.h,v 1.1.1.1 1999/02/14 16:18:15 hgs Exp $
+ *  $Id: utilnt.h,v 1.3 2002/09/10 10:20:59 at541 Exp $
  */
 
 /*
@@ -90,23 +97,23 @@ struct passwd *getpwnam(const char *name);
  * stuff.
  */
 struct lc_time_T {
-	const char *	mon[12];
-	const char *	month[12];
-	const char *	wday[7];
-	const char *	weekday[7];
-	const char *	X_fmt;
-	const char *	x_fmt;
-	const char *	c_fmt;
-	const char *	am;
-	const char *	pm;
-	const char *	date_fmt;
+  const char *  mon[12];
+  const char *  month[12];
+  const char *  wday[7];
+  const char *  weekday[7];
+  const char *  X_fmt;
+  const char *  x_fmt;
+  const char *  c_fmt;
+  const char *  am;
+  const char *  pm;
+  const char *  date_fmt;
 };
 
-struct lc_time_T _time_localebuf;
-int _time_using_locale;
-const struct lc_time_T _C_time_locale;
+//struct lc_time_T _time_localebuf;
+//int _time_using_locale;
+//const struct lc_time_T _C_time_locale;
 
-#define Locale	(_time_using_locale ? &_time_localebuf : &_C_time_locale)
+#define Locale  (_time_using_locale ? &_time_localebuf : &_C_time_locale)
 
 char *strptime(const char *buf, const char *format, struct tm *tm);
 
