@@ -2,7 +2,7 @@
 * RTP translator.
 *
 * Usage:
-*   rtptrans host/port/ttl host/port/ttl [...]
+*   rtptrans [host]/port[/ttl] [host]/port[/ttl] [...]
 *
 * Forwards RTP/RTCP packets from one of the named sockets to all
 * others.  Addresses can be a multicast or unicast.  TTL values for
@@ -143,7 +143,7 @@ int create_stream(int addr, int next)
     last=new_stream;
     return new_stream->seq;
   }
-    
+
   for(elem=head;elem->next!=NULL;elem=elem->next) {
     if(elem->next->addr>addr) {
       new_stream->next=elem->next;
@@ -232,12 +232,14 @@ static Notify_value socket_handler(Notify_client client, int sock)
   /* do not translate packets that already use RTP or arrive over the unicast
    link*/
   if ((rtp_hdr->version==2)||((sock!=multi_sock[0])&&(sock!=multi_sock[1]))) {
-    rtp_hdr=(rtp_hdr_t *)packet;   
+    rtp_hdr=(rtp_hdr_t *)packet;
     for (i = 0; i < hostc; i++) {
-      if (side[i][proto].sock != sock) {
+      if (side[i][proto].sock != sock
+          && side[i][proto].sin.sin_addr.s_addr != INADDR_ANY) {
         if (sendto(side[i][2].sock, packet, len, 0,
           (struct sockaddr *)&side[i][proto].sin,sizeof(side[i][proto].sin))<0)
-        perror("sendto RTCP");
+//        perror("sendto RTCP");
+          ;
       }
     }
   }
@@ -293,9 +295,9 @@ static Notify_value socket_handler(Notify_client client, int sock)
 
 #if defined(Linux) || defined(WIN32)
       /* 
-	   * Stupid little Linux and stupid big Win32 does not support
-	   * sendmsg(), thus, use copying instead; contributed by Lutz
-	   * Grueneberg <gruen@rvs.uni-hannover.de>.
+       * Stupid little Linux and stupid big Win32 does not support
+       * sendmsg(), thus, use copying instead; contributed by Lutz
+       * Grueneberg <gruen@rvs.uni-hannover.de>.
        */
       {
         unsigned char mbuf[10000];
@@ -324,7 +326,7 @@ static Notify_value socket_handler(Notify_client client, int sock)
         if (side[i][proto].sock != sock) {
           msg.msg_name = (caddr_t ) &side[i][proto].sin;
           msg.msg_namelen = sizeof(side[i][proto].sin);
-#if defined(__FreeBSD__) || defined(__linux__) /* Or presumably other BSD 4.4 systems */
+#if defined(__FreeBSD__) || defined(__linux__) || defined(__darwin__) /* Or presumably other BSD 4.4 systems */
           msg.msg_control = 0;
           msg.msg_controllen = 0;
 #else
@@ -404,9 +406,9 @@ static Notify_value socket_handler(Notify_client client, int sock)
 void usage(char *argv0)
 {
   fprintf(stderr, 
-"Usage: %s \
-[address]/port/ttl\
-[address]/port/ttl [...]\n",
+"Usage: %s\
+ host/port[/ttl]\
+ host/port[/ttl] [...]\n",
  argv0);
 }
 
@@ -442,13 +444,14 @@ int main(int argc, char *argv[])
     }
   }
 
-  if (optind == argc) {
+  if (argc - optind < 2) {
+    fprintf(stderr, "%s: Requires two host/port[/tll].\n", argv[0]);
     usage(argv[0]);
     exit(1);
   }
 
   /* Parse host descriptions. */
-  for (i = 0; i < argc-optind; i++) {
+  for (i = 0; i < argc - optind; i++) {
     char *s;
 
     if (i >= MAX_HOST) break;
@@ -467,7 +470,8 @@ int main(int argc, char *argv[])
       port = atoi(s+1);
       if (port & 1) {
         fprintf(stderr, "%s: Port must be even.\n", argv[0]);
-        /* exit(1); */
+        usage(argv[0]);
+        exit(1);
       }
       host[i].sin.sin_port = htons(port);
       s = strchr(s+1, '/');
@@ -476,6 +480,11 @@ int main(int argc, char *argv[])
       }
     }
     host[i].sin.sin_addr = host2ip(host[i].name);
+    if (host[i].sin.sin_addr.s_addr == -1) {
+      fprintf(stderr, "%s: Invalid host. %s\n", argv[0], host[i].name);
+      usage(argv[0]);
+      exit(1);
+    }
     if (IN_CLASSD(ntohl(host[i].sin.sin_addr.s_addr))) {
       host[i].mreq.imr_multiaddr        = host[i].sin.sin_addr;
       host[i].mreq.imr_interface.s_addr = htonl(INADDR_ANY);
